
% model parameters:
theta_init = -0.05; % initial angle of pole (in radians)
x_init = 0.2;       % initial cart position (in m)
g = 9.81;           % gravitational acceleration (in m/s^2)
m_c = 1;            % mass of cart (in kg)
m_p = 0.1;          % mass of pole (in kg)
l = 0.5;            % distance from pivot to pole center (length = 2*l)
h_max = 2.4;        % maximal deviation in cart position (in m)
r_max = 0.209;      % maximal deviation in pole angle (in radians=approx. 12)
tau = 0.02;         % time step (in s)

% controller parameters (linear controller):
F_m = 0.1; % fixed force constant
k1 = 1;
k2 = 0.1;
k3 = 1;
k4 = 1;

% simulation parameters:
simulationTime = 120; % (in s)
numTimeSteps = simulationTime/tau;

% initial values:
angle_theta = theta_init;
angle_velo = 0;
angle_acc = 0;
pos_x = x_init;
pos_velo = 0;
pos_acc = 0;
F = 0;

for t = 1:numTimeSteps
    
    angle_acc = getAngleAccelaration(g, angle_theta, angle_velo, F, m_p, m_c, l);
    pos_acc = getPosAccelaration(F, m_p, l, angle_velo, angle_theta, angle_acc, m_c);
    
    % integration step of Euler method:
    angle_theta = angle_theta + tau.*angle_velo;
    angle_velo = angle_velo + tau.*angle_acc;
    pos_x = pos_x + tau.*pos_velo;
    pos_velo = pos_velo + tau.*pos_acc;
    
    % check whether current state is feasible and stop if not:
    if (pos_x < -h_max || pos_x > h_max || ...
            angle_theta < -r_max || angle_theta > r_max)
        fprintf('Current state of pole not stable after %e seconds\n', t*tau);
        fprintf('   x_%d = %e; theta_%d = %e\n', t, pos_x, t, angle_theta);
        break;
    end
        
    % compute response:
    F = linearController(F_m, k1, k2, k3, k4, pos_x, pos_velo, angle_theta, angle_velo);
    
    fprintf('iteration %d\n', t);
    fprintf('---------------\n');
    fprintf(' x               = %e\n', pos_x);
    fprintf(' dx/dt           = %e\n', pos_velo);
    fprintf(' d^2x/dt^2       = %e\n', pos_acc);
    fprintf(' theta           = %e\n', angle_theta);
    fprintf(' d(theta)/dt     = %e\n', angle_velo);
    fprintf(' d^2(theta)/dt^2 = %e\n', angle_acc);
    fprintf('\n');
    
end